/*
    Separator Shape
        あかつきみさき

    このスクリプトについて
        空のシェイプを追加し,ガイド設定とシャイ設定するスクリプト.
        タイムラインパネルの区切りとして使用すると便利です.
        キャッシュを侵食しないように表示はせず,マイナス地点に配置されます.
        レイヤーを選択している場合,選択したレイヤーの上に追加されます.

    使用方法
        ファイル→スクリプト→スクリプトファイルを実行.

    動作環境
        Adobe After Effects 2022以上

    バージョン情報
        2021/12/22 Ver 1.0.0 Release
*/

/// <reference path="D:\lib\aftereffects.d.ts\ae.d.ts"/>

(function() {
    var actComp = app.project.activeItem;
    if (actComp == null || !(actComp instanceof CompItem)) {
        return 0;
    }

    var selLayers = actComp.selectedLayers;
    var layerIdList = [];
    var existsTrackMatteLayer = false;
    for (var i = actComp.layers.length; 0 < i; i--) {
        if (!actComp.layer(i).selected) {
            continue;
        }

        if (actComp.layer(i).trackMatteType != TrackMatteType.NO_TRACK_MATTE) {
            existsTrackMatteLayer = true;
        }

        layerIdList[layerIdList.length] = actComp.layer(i).id;
        actComp.layer(i).selected = false;
    }

    if (selLayers.length == 0) {
        app.beginUndoGroup("Separator Shape");
        var addedShapeLayer = actComp.layers.addShape();
        addedShapeLayer.name = "Separator Shape";
        addedShapeLayer.comment = "Separator Shape";
        addedShapeLayer.guideLayer = true;
        addedShapeLayer.shy = true;
        addedShapeLayer.enabled = false;
        addedShapeLayer.inPoint = -actComp.frameDuration;
        addedShapeLayer.outPoint = 0;
        app.endUndoGroup();

        if (existsTrackMatteLayer) {
            alert("トラックマットが設定されたレイヤーに対して実行されました.");
        }
        return 0;
    }

    app.beginUndoGroup("Separator Shape");

    for (var j = 0; j < layerIdList.length; j++) {
        for (var i = actComp.layers.length; 0 < i; i--) {
            if (actComp.layer(i).id != layerIdList[j]) {
                continue;
            }

            var addedShapeLayer = actComp.layers.addShape();
            addedShapeLayer.name = "Separator - " + actComp.layer(i + 1).name;
            addedShapeLayer.comment = "Separator Shape";
            addedShapeLayer.guideLayer = true;
            addedShapeLayer.shy = true;
            addedShapeLayer.enabled = false;
            addedShapeLayer.selected = false;
            addedShapeLayer.inPoint = -actComp.frameDuration;
            addedShapeLayer.outPoint = 0;
            if (actComp.layer(i).index != 1) {
                addedShapeLayer.moveAfter(actComp.layer(i));
            }
        }
    }

    app.endUndoGroup();

    if (existsTrackMatteLayer) {
        alert("トラックマットが設定されたレイヤーに対して実行されました.");
    }
    return 0;
}).call(this);